# makeimg

[![builds.sr.ht status](https://builds.sr.ht/~bitfehler/makeimg/commits.svg)](https://builds.sr.ht/~bitfehler/makeimg/commits?)

Build system images from declarative input.

`makeimg` provides a declarative way to describe system images (e.g. VM
images). The project was initially inspired by the [OpenWRT image
builder](https://openwrt.org/docs/guide-user/additional-software/imagebuilder)
and Arch Linux's [PKGBUILD](https://wiki.archlinux.org/title/PKGBUILD) format.
It was then convinced by Alpine's
[APKBUILD](https://wiki.alpinelinux.org/wiki/APKBUILD_Reference) to become
POSIX-compliant.

* [Project homepage](https://sr.ht/~bitfehler/makeimg)
* [Source code](https://git.sr.ht/~bitfehler/makeimg)

Some examples can be found in the `examples` folder.

`makeimg` is a stand-alone project and quite useful on its own, but it is
developed with a focus on its part in project
[Anemos](https://sr.ht/~bitfehler/anemos/). Check it out to learn how to use
`makeimg` to deploy bare-metal hosts!

**NOTE:** The expected project structure has changed from version 0.7 to 0.8 -
however, most projects can be converted automatically with a script found in
the `contrib` folder.

## Documentation

`makeimg` comes with extensive man pages, one for the tool and one for the
format of the `IMGBUILD` file that describes the images. You can read the
online version here:
[makeimg.1](https://anemos.io/man/makeimg.1.html),
[IMGBUILD.5](https://anemos.io/man/IMGBUILD.5.html)
or build the man pages yourself (see below).

## Status

The current version of `makeimg` should technically run on any reasonable Linux
system. It can create Arch Linux, Alpine Linux, and Debian Linux images.
However, to do so, it depends on `pacman`, `apk`, and `debootstrap`
respectively. Arch Linux and Alpine Linux offer all three, and are hence best
suited for running `makeimg`. On Debian, you will only be able to create Debian
images (unless the respective tools have been manually installed and set up).

## Requirements

The required tools depend heavily on the target system and the type of image
being built. A properly configured `pacman` or `apk` is required to build Arch
or Alpine images respectively. For building disk images, `parted` is needed.
For more details, check the source code for now. Packages coming soon.

## Installing

For now, I only provide an [AUR
package](https://aur.archlinux.org/packages/makeimg). Or just copy the script
somewhere into your `$PATH`.

## Feedback

Got questions or other feedback? Please use the [anemos mailing
list][anemos-ml].

## Development

The script can be used as is. Linting and static analysis is performed with
[shfmt](https://github.com/mvdan/sh) and
[shellcheck](https://www.shellcheck.net) respectively. When submitting patches,
please make sure `make check` still succeeds.

To build the man pages, [scdoc](https://sr.ht/~sircmpwn/scdoc) is required.
You can also build a HTML version of the man pages with
[scd2html](https://sr.ht/~bitfehler/scd2html).

* Man pages: `make man` 
* Man pages (html): `make html`

Please send patches to the [anemos mailing list][anemos-ml].

[anemos-ml]: https://lists.sr.ht/~bitfehler/anemos
